from django.http import HttpResponseForbidden
from django.shortcuts import get_object_or_404
from .models import EmailVerification

class UserVerifiedMixin:
    def verify_user_email(self, request, *args, **kwargs):
        user = request.user
        if user.is_authenticated:
            email_verification = get_object_or_404(EmailVerification,user=user)
            if email_verification.is_verified:
                return None
        return HttpResponseForbidden("User email not verified.")

    def dispatch(self, request, *args, **kwargs):
        response = self.verify_user_email(request, *args, **kwargs)
        if response:
            return response # If there's an issue, return the forbidden response
        return super().dispatch(request, *args, **kwargs)


from django.db import models
from django.contrib.auth import get_user_model
from django.contrib.auth.models import BaseUserManager, AbstractBaseUser
from .email_utils import send_verification_email_wrapper


class EmailVerification(models.Model):
    user = models.OneToOneField(get_user_model(), on_delete=models.CASCADE)
    is_verified = models.BooleanField(default=False)
    email_sent_status = models.BooleanField(default=False)
    last_email_date = models.DateTimeField(null=True)
    verified_date = models.DateTimeField(null=True)
    
    def send_email(self,request=None,domain=None):
        send_verification_email_wrapper(self.user,request,domain)


from django.conf import settings
from django.db.models.signals import post_save, pre_save
from django.utils.timezone import now
from django.dispatch import receiver
from .models import EmailVerification
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError

@receiver(post_save, sender=get_user_model())
def create_email_verification(sender, instance=None, created=False, **kwargs):
    if created and not instance.is_superuser:
        ev = EmailVerification.objects.create(user=instance)
        try:
            ev.send_email()
            ev.last_email_date = now()
            ev.email_sent_status = True
            ev.save()
        except Exception as e:
            ev.last_email_date = now()
            ev.email_sent_status = False
            ev.save()
            raise e
            


@receiver(post_save, sender=get_user_model())
def save_email_verification(sender, instance=None, **kwargs):
    if not instance.is_superuser:
        instance.emailverification.save()


def check_email(sender, instance, **kwargs):
    email = instance.email
    if email and sender.objects.filter(email=email).exclude(pk=instance.pk).exists():
        raise ValidationError('Email Exists')


if getattr(settings, 'EMAIL_VERIFY_ENFORCE_UNIQUE_EMAILS', True):
    pre_save.connect(check_email, get_user_model())



# import os
# import environ
# from django.test import TestCase
# from django.conf import settings
# from django.contrib.auth import get_user_model
# from .models import EmailVerification
# from .forms import EmailVerificationUserCreationForm

# ## for custom user testing create custom_user app and define custom user model in its models.py
# ## make sure to install it before email_verify

# def simulate_email_send(user, link):
#     print(f"mail sent to {user.email} with link: {link}")
    
# ### PASSED ###
# class EmailVerificationTestCase(TestCase):
#     def setUp(self):
#         settings.EMAIL_VERIFY_SEND_FUNC = simulate_email_send
#         User = get_user_model()
#         self.user = User.objects.create_user('testUser','test@example.com', 'testpassword')
        
#     def test_email_verification_creation(self):
#         # Test that EmailVerification object is created for the user
#         email_verification = EmailVerification.objects.get(user=self.user)
#         self.assertIsNotNone(email_verification)
#         self.assertFalse(email_verification.is_verified)

#     def test_simulated_email_send(self):
#         # Test that the simulated email send function works
#         email_verification = EmailVerification.objects.get(user=self.user)
#         email_verification.send_email()  # This should print the simulated email message

#     def test_user_creation_form(self):
#         # Test the custom user creation form
#         form_data = {'email': 'newuser@example.com', 'password1': 'testpass123', 'password2': 'testpass123'}
#         form = EmailVerificationUserCreationForm(data=form_data)
#         self.assertTrue(form.is_valid())
#         user = form.save()
#         self.assertIsNotNone(user)
#         self.assertEqual(user.email, 'newuser@example.com')

# ### PASSED ###    
# class EmailVerificationSMTPTestCase(TestCase):
#     def setUp(self):
#         environ.Env()
#         environ.Env.read_env()
        
#         settings.DEBUG = True
#         settings.EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
#         settings.EMAIL_HOST = os.environ.get('EMAIL_HOST')
#         settings.EMAIL_HOST_USER = os.environ.get('EMAIL_HOST_USER')
#         settings.EMAIL_HOST_PASSWORD = os.environ.get('EMAIL_HOST_PASSWORD')
#         settings.EMAIL_PORT = 587
#         settings.EMAIL_USE_TLS = True
#         settings.EMAIL_VERIFY_FROM_ADDRESS = os.environ.get('EMAIL_VERIFY_FROM_ADDRESS')
#         User = get_user_model()
#         self.user = User.objects.create_user('newUser',os.environ.get('EMAIL_VERIFY_FROM_ADDRESS'), 'testpassword')
#     def test_email_verification_creation(self):
#         # Test that EmailVerification object is created for the user
#         email_verification = EmailVerification.objects.get(user=self.user)
#         self.assertIsNotNone(email_verification)
#         self.assertFalse(email_verification.is_verified)
        
#     def test_user_creation_form(self):
#         # Test the custom user creation form
#         form_data = {'email': os.environ.get('EMAIL_VERIFY_FROM_ADDRESS'), 'password1': 'testpass123', 'password2': 'testpass123'}
#         form = EmailVerificationUserCreationForm(data=form_data)
#         self.assertTrue(form.is_valid())
#         user = form.save()
#         self.assertIsNotNone(user)
#         self.assertEqual(user.email, os.environ.get('EMAIL_VERIFY_FROM_ADDRESS'))


from django.urls import path
from . import views

app_name='email_verify'
urlpatterns = [
    path('verify_email/<str:token>/', views.EmailVerificationView.as_view(), name='verify_email'),
    path('email_verify/fail/',views.failure_view,name="verification_failure"),
    path('email_verify/success/',views.success_view,name="verification_success"),
]


from django.apps import AppConfig


class EmailVerifyConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'email_verify'
    def ready(self):
        import email_verify.signals


class TokenExpired(Exception):
    def __init__(self, message = 'The token has expired') -> None:
        self.message = message
        super().__init__(self.message)
    
    def __str__(self):
        return self.message
    
class TokenGenerationException(Exception):
    def __init__(self, message = 'Token could not be generated') -> None:
        self.message = message
        super().__init__(self.message)
    
    def __str__(self):
        return self.message
    
class EmailSendingException(Exception):
    def __init__(self, message = 'Email wasn\'t sent') -> None:
        self.message = message
        super().__init__(self.message)
    
    def __str__(self):
        return self.message
    
class InvalidDomainException(Exception):
    def __init__(self, message = 'The token\'s domain does not match any of the allowed hosts') -> None:
        self.message = message
        super().__init__(self.message)
    
    def __str__(self):
        return self.message
    
class AlreadyVerifiedException(Exception):
    def __init__(self, message = 'This account has already been verified') -> None:
        self.message = message
        super().__init__(self.message)
    
    def __str__(self):
        return self.message


from functools import wraps
from django.http import HttpResponseForbidden
from django.shortcuts import get_object_or_404
from .models import EmailVerification

def user_verified(view_func):
    @wraps(view_func)
    def _wrapped_view(request, *args, **kwargs):
        user = request.user
        if user.is_authenticated:
            email_verification = get_object_or_404(EmailVerification, user=user)
            if email_verification.is_verified:
                return view_func(request, *args, **kwargs)
        return HttpResponseForbidden("User email not verified.")

    return _wrapped_view


from django.utils.timezone import now
from itsdangerous.url_safe import URLSafeTimedSerializer
from django.conf import settings
import json

def generate_token(user, domain=None):
    domain = domain or getattr(settings,'EMAIL_VERIFY_USE_DOMAIN',None)
    if domain is None and not settings.DEBUG:
        raise ValueError("Domain must be provided in production environment.")
    s = URLSafeTimedSerializer(settings.SECRET_KEY)
    str = s.dumps(json.dumps({'user_id': user.id, 'domain': domain}))
    return str


def verify_token(token):
    from .models import EmailVerification
    from itsdangerous import SignatureExpired, BadSignature
    s = URLSafeTimedSerializer(settings.SECRET_KEY)
    expires = getattr(settings, 'EMAIL_VERIFY_EXPIRES_IN', 3600)
    expires = expires if expires > 0 else None
    try:
        str = s.loads(token, max_age=expires)
        data = json.loads(str)
    except SignatureExpired:
        return (False, "Token has expired.")
    except BadSignature:
        return (False, "Invalid token.")

    domain = data.get('domain')
    user_id = data.get('user_id')
    
    ev_allowed_host = getattr(settings,'EMAIL_VERIFY_ALLOWED_HOST',None)
    domain_valid = domain in settings.ALLOWED_HOSTS or (ev_allowed_host and domain == ev_allowed_host)
    
    if not settings.DEBUG and domain_valid:
        return (False, "Invalid domain.")

    if user_id:
        try:
            email_verification = EmailVerification.objects.get(user__id=user_id)
            if email_verification.is_verified:
                return (False, "Already verified.")
            
            email_verification.is_verified = True
            email_verification.verified_date = now()
            email_verification.save()
            return (True, None)
        except EmailVerification.DoesNotExist:
            return (False, "Email verification does not exist.")

    return (False, "Unknown error.")


import logging
import smtplib
from django.contrib.sites.shortcuts import get_current_site
from django.urls import reverse
from django.conf import settings
from .utils import generate_token
from .exceptions import TokenGenerationException, EmailSendingException, InvalidDomainException


def default_send_email(user, verification_link):
    from django.core.mail import send_mail
    from django.conf import settings
    subject = getattr(settings, 'EMAIL_VERIFY_SUBJECT_LINE','Email Verification')
    message = getattr(settings, 'EMAIL_VERIFY_TEXT_MESSAGE','If you can\'t display the link, copy and paste this into your browser\'s address bar: $:_VERIFICATION_LINK')
    html_message = getattr(settings, 'EMAIL_VERIFY_HTML_MESSAGE',f'Verify your email by clicking the link: <br><a href="$:_VERIFICATION_LINK" style="background-color: #2196F3; color: white; text-align: center; padding: 8px 15px; text-decoration: none; display: inline-block; font-family: Arial, sans-serif; font-size: 16px; border-radius: 4px;">Verify your email</a><br>If you can\'t display the link copy and paste this into your browser\'s address bar: $:_VERIFICATION_LINK')
    from_email = getattr(settings,'EMAIL_VERIFY_FROM_ADDRESS','verify@email_verify.com')
    recipient_list = [user.email]
    send_mail(subject, message.replace('$:_VERIFICATION_LINK',verification_link), from_email, recipient_list, html_message=html_message.replace('$:_VERIFICATION_LINK',verification_link))
    
def send_verification_email(user, send_email_func, request=None,domain=None):
    if domain is None and getattr(settings,'EMAIL_VERIFY_USE_DOMAIN',None) is None:
        # Use the request object if provided, otherwise fall back to ALLOWED_HOSTS
        if request:
            current_site = get_current_site(request)
            domain = current_site.domain
            port = request.get_port()
            if port not in ('80', '443'):
                domain += f":{port}"
        else:
            if settings.DEBUG:
                domain = settings.ALLOWED_HOSTS[0] if settings.ALLOWED_HOSTS else 'localhost'
                port = settings.DEBUG_PORT if hasattr(settings, 'DEBUG_PORT') else '8000'
                domain += f":{port}"
            elif settings.ALLOWED_HOSTS:
                domain = settings.ALLOWED_HOSTS[0]
            else:
                raise InvalidDomainException("Couldn't resolve domain for token. If DEBUG is set to false ALLOWED_HOSTS[0] must be set or request object should be passed to function send_verification_email")
    elif not domain:
        domain = getattr(settings,'EMAIL_VERIFY_USE_DOMAIN',None)
        if not domain:
            raise InvalidDomainException("Couldn't resolve domain for token.")
    
    try:
        token = generate_token(user, domain=domain)
    except Exception as e:
        logging.error(f"Problem generating token: {e}")
        raise TokenGenerationException("An error occurred while generating token") from e

    if request:
        verification_link = request.build_absolute_uri(reverse('email_verify:verify_email', args=[token]))
    else:
        path = reverse('email_verify:verify_email', args=[token])
        protocol = 'https' if settings.SECURE_SSL_REDIRECT else 'http'
        verification_link = f"{protocol}://{domain}{path}"

    try:
        send_email_func(user, verification_link)
    except smtplib.SMTPException as e:
        logging.error(f"SMTP error occurred: {e}")
        raise EmailSendingException("An SMTP error occurred while sending email.") from e
    except Exception as e:
        # General catch-all for errors, especially if a user-provided email sending function is used
        logging.error(f"General email sending error occurred: {e}")
        raise EmailSendingException("An error occurred while sending email. Please refer to the logs for details.") from e

    
def send_verification_email_wrapper(user,request=None,domain=None):
    send_fn = getattr(settings,'EMAIL_VERIFY_SEND_FUNC', default_send_email)
    send_verification_email(user,send_fn,request,domain)


from django.conf import settings
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse_lazy
from django.views import View
from .utils import verify_token

class EmailVerificationView(View):
    success_url = reverse_lazy('email_verify:verification_success')
    failure_url = reverse_lazy('email_verify:verification_failure')

    def get(self, _, token):
        success, message = verify_token(token)
        
        if success:
            return HttpResponseRedirect(self.success_url)
        else:
            return HttpResponseRedirect(self.failure_url + f"?message={message}")
        
def failure_view(request):
    message = request.GET.get('message', 'An error occurred.')
    return render(request, 'email_verify/failure.html', {'message': message})

def success_view(request):
    main_page = getattr(settings,'MAIN_PAGE','/')
    return render(request, 'email_verify/success.html',{'main_page':main_page})


from setuptools import setup, find_packages

with open('README.md', 'r') as readme_file:
    long_description = readme_file.read()

setup(
    name='email_verify',
    version='0.1.8',
    author='can gologlu',
    author_email='can@xn--glolu-jua30a.com',
    description='Django app for e-mail verification on sign up',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'itsdangerous>=2.1.2',
    ],
)


from collections import OrderedDict
from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import UserCreationForm
from django.core.exceptions import ValidationError
from django.core.validators import EmailValidator

class EmailVerificationUserCreationForm(UserCreationForm):
    class Meta:
        model = get_user_model()
        fields = ('email',)
        
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        User = get_user_model()
        if 'username' in [field.name for field in User._meta.fields]:
            self.fields['username'] = forms.CharField(max_length=30)
            new_order = OrderedDict()
            new_order['username'] = self.fields['username']
            for key, value in self.fields.items():
                if key != 'username':
                    new_order[key] = value
            self.fields = new_order

    def clean_email(self):
        email = self.cleaned_data.get('email', None)
        if email:
            try:
                EmailValidator(message="Invalid e-mail")(email)
            except ValidationError as e:
                raise ValidationError("Invalid e-mail")
        return email

    def save(self, commit=True, request=None):
        user = super().save(commit=False)
        if commit:
            user.save()
        return user